/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.ImageDiffer.ResultWriters;

import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



import org.gfxtest.ImageDiffer.ComparisonResult;
import org.gfxtest.ImageDiffer.Configuration;
import org.gfxtest.framework.Log;

/**
 * 
 *
 * @author Pavel Tisnovsky
 */
public class HtmlStructureWriter
{
    /**
     * Default encoding used for all input/output operations.
     */
    private static final String DEFAULT_ENCODING = "UTF-8";

    private static final String TEMPLATE_DIR = "templates";

    static Log log = new Log(HtmlStructureWriter.class.getName(), true);

    public static void createAndFillResultDirectory(Configuration configuration, BufferedImage[] sourceImages,
            ComparisonResult comparisonResult, String dirName) throws IOException
    {
        File newDir = new File(configuration.getOutputDirectory(), dirName);
        log.logProcess("creating directory %s", newDir.getAbsolutePath());
        if (!newDir.mkdir())
        {
            log.logError("error creating directory %s", newDir.getAbsolutePath());
        }
        if (!comparisonResult.isEqualsImages())
        {
            log.logProcess("writing source and diff images to directory %s", dirName);
            //ImageUtils.writeImage(sourceImages[0], newDir, "source1.png");
            //ImageUtils.writeImage(sourceImages[1], newDir, "source2.png");
            //ImageUtils.writeImage(comparisonResult.getDiffImage(), newDir, "diff.png");
        }
        else
        {
            log.logProcess("writing source image to directory %s", dirName);
            //ImageUtils.writeImage(sourceImages[0], newDir, "source1.png");
        }
        createHtmlFile(newDir, comparisonResult);
    }

    private static void createHtmlFile(File newDir, ComparisonResult cr) throws IOException
    {
        String templateName = cr.isEqualsImages() ? "template_same_images.html" : "template_different_images.html";
        BufferedWriter writer = null;
        BufferedReader template = null;
        try
        {
            writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(new File(newDir, "result.html")), DEFAULT_ENCODING));
            template = new BufferedReader(new InputStreamReader(new FileInputStream(new File(TEMPLATE_DIR, templateName)), DEFAULT_ENCODING));
            String line;
            while ((line = template.readLine()) != null)
            {
                writer.write(replacePlaceholders(cr, newDir, line));
            }
        }
        catch (UnsupportedEncodingException e)
        {
            e.printStackTrace();
            throw e;
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
            throw e;
        }
        catch (IOException e)
        {
            e.printStackTrace();
            throw e;
        }
        finally
        {
            if (template != null)
            {
                try
                {
                    template.close();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
            if (writer != null)
            {
                try
                {
                    writer.close();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
        }
    }

    private static String replacePlaceholders(ComparisonResult cr, File newDir, String line)
    {
        StringBuffer out = new StringBuffer();
        Pattern pattern = Pattern.compile("\\$\\{(.*?)\\}", Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(line);

        while (matcher.find())
        {
            matcher.appendReplacement(out, getPlaceholderValue(cr, matcher.group(1)));
        }
        matcher.appendTail(out);
        return out.toString() + "\n";
    }

    private static String getPlaceholderValue(ComparisonResult cr, String group)
    {
        Method method;
        try
        {
            // call appropriate getter
            method = cr.getClass().getDeclaredMethod("get" + group, (Class<?>[]) null);
            return "" + method.invoke(cr, (Object[]) null);
        }
        catch (IllegalAccessException e)
        {
            // print ST in case of exception
            e.printStackTrace();
        }
        catch (SecurityException e)
        {
            // print ST in case of exception
            e.printStackTrace();
        }
        catch (NoSuchMethodException e)
        {
            // print ST in case of exception
            e.printStackTrace();
        }
        catch (IllegalArgumentException e)
        {
            // print ST in case of exception
            e.printStackTrace();
        }
        catch (InvocationTargetException e)
        {
            // print ST in case of exception
            e.printStackTrace();
        }
        return group;
    }

}
